#コミック
class Comic < ActiveRecord::Base
  has_many :stories
  belongs_to :author
  
  validates :title, :presence => true, :length => {:maximum => 100}
  validates :visible, :presence => true, :numericality => true, :inclusion => {:in => 0..1}
  
  def supply_default
    self.visible = 0 if self.visible.blank?
  end
  
  def overwrite au
    return false unless au
    self.author_id = au.id
  end
  
  def own? roles
    roles = [roles] unless roles.respond_to?(:each)
    au = Comic.get_author_from_roles roles
    return false unless au
    self.author_id == au.id
  end
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.own?(roles)
    self.visible > 0
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.default_panel_size
    30
  end
  
  def self.max_panel_size
    200
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(Comic.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => ['visible > 0'], :order => 'updated_at desc'})
    Comic.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:stories => {:panel => {}}, :author => {}}}
  end
  
  def self.list_json_opt
    {:include => {:stories => {:include => {:panel => {}}}, :author => {}}}
  end
  
  def self.mylist au, page = 1, page_size = Author.default_comic_page_size
    opt = {}
    opt.merge!(Comic.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => ['comics.author_id = ?', au.id], :order => 'comics.updated_at desc'})
    Comic.find(:all, opt)
  end
  
  def self.show cid, roles
    opt = {}
    opt.merge!(Comic.show_opt)
    res = Comic.find(cid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(roles)
    res
  end
  
  def self.edit cid, au
    opt = {}
    opt.merge!(Comic.show_opt)
    res = Comic.find(cid, opt)
    raise ActiveRecord::Forbidden unless res.own?(au)
    res
  end
  
  def self.show_opt
    {:include => {:stories => {:panel => {}}, :author => {}}}
  end
  
  def self.show_json_opt
    {:include => {:stories => {:include => {:panel => {}}}, :author => {}}}
  end
  
  def self.visible_count
    Comic.count 'visible > 0'
  end
  
  def destroy_with_story
    res = false
    Comic.transaction do
      self.stories.each do |story|
        raise ActiveRecord::Rollback unless story.destroy
      end
      raise ActiveRecord::Rollback unless self.destroy
      res = true
    end
    res
  end
  
end
