#実素材
class Picture < ActiveRecord::Base
  belongs_to :original_picture
  belongs_to :license
  belongs_to :artist
  has_one :resource_picture
  
  validates :original_picture_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :revision, :presence => true, :numericality => true
  validates :ext, :presence => true, :length => {:maximum => 4}, :inclusion => {:in => ['png', 'jpeg', 'gif']}
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :filesize, :presence => true, :numericality => {:greater_than => 0, :less_than_or_equal_to => 2000000}, :natural_number => true
  validates :md5, :presence => true, :length => {:minimum => 32, :maximum => 32}
  validates :license_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :artist_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :artist_name, :presence => true
  validates :classname, :presence => true, :length => {:maximum => 50}
  
  def supply_default
  end
  
  def overwrite rp
    attr = {:width => rp.width, :height => rp.height, :ext => rp.ext, :filesize => rp.filesize, 
      :original_picture_id => rp.original_picture_id, :license_id => rp.license_id, :artist_id => rp.artist_id, 
      :md5 => rp.md5, :artist_name => rp.artist_name, :classname => rp.classname, :credit => rp.credit, :settings => rp.settings
    }
    self.attributes = attr
    self.revision = self.new_revision   #Do not move to attr. new_revision reffernces self.original_picture_id
  end
  
  def own? roles
    roles = [roles] unless roles.respond_to?(:each)
    ar = Picture.get_artist_from_roles roles
    return false unless ar
    self.artist_id == ar.id
  end
  
  def visible? ar
    return true
  end
  
  def showable? au = nil
    return false unless self.original_picture
    return true if self.own?(au)
    self.enable? and self.head?
  end
  
  def filename
    "#{self.id}.#{self.ext}"
  end
  
  def gifname
    "#{self.id}.gif"
  end
  
  def mime_type
    "image/#{self.ext}"
  end
  
  def url
    '/pictures/' + filename
  end
  
  def opt_img_tag
    {:src => self.url, :width => self.width, :height => self.height}
  end
  
  def tmb_opt_img_tag
    tw, th = PettanImager.thumbnail_size(self.width, self.height)
    {:src => self.url, :width => tw, :height => th}
  end
  
  def tail_opt_img_tag img
    {:src => img, :width => self.width, :height => self.height}
  end
  
  def tail_tmb_opt_img_tag img
    tw, th = PettanImager.thumbnail_size(self.width, self.height)
    {:src => img, :width => tw, :height => th}
  end
  
  def new_revision
    Picture.maximum(:revision, :conditions => ['original_picture_id = ?', self.original_picture_id]).to_i + 1
  end
  
  def enable?
    r = self.head.resource_picture
    r ? true : false
  end
  
  def self.head opid
    Picture.find(:first, :conditions => ['original_picture_id = ?', opid], :order => 'pictures.revision desc')
  end
  
  def head
    Picture.head(self.original_picture_id)
  end
  
  def head?
    self == head
  end
  
  def to_gif?
    self.ext == 'png' and self.flag_gif_convert >= 0
  end
  
  def subdirs
    flag_reverse < 0 ? [''] : ['', 'v', 'h', 'vh']
  end
  
  def self.find_by_md5 md5
    r = Picture.find :all, :conditions => ['pictures.md5 = ?', md5], :order => 'pictures.updated_at desc'
  end
  
  def self.list_by_md5 md5, opid = nil
    cond = if opid.blank?
      ['pictures.md5 = ?', md5]
    else
      ['pictures.md5 = ? and pictures.original_picture_id <> ?', md5, opid]
    end
    r = Picture.find :all, :conditions => cond, :order => 'pictures.updated_at desc'
  end
  
  def self.exist_by_md5 md5, opid
    Picture.list_by_md5(md5, opid).empty? ? false : true
  end
  
  def self.show rid, roles
    opt = {}
    r = Picture.find(rid, opt)
    raise ActiveRecord::Forbidden unless r.visible?(roles)
    r
  end
  
  def store(imager)
    res = false
    if res = self.save
      if res = self.store_picture(imager, self.filename)
        if self.to_gif?
          if gifimager = imager.to_gif
            res = self.store_picture(gifimager, self.gifname)
          else
            res = false
          end
        end
      end
    end
    res
  end
  
  def store_picture(imager, fn)
    res = false
    subdirs.each do |d|
      picdata = d.empty? ? imager.binary : imager.__send__(d)
      res = PictureIO.picture_io.put(picdata, fn, d)
      break unless res
    end
    res
  end
  
  def restore(subdir = nil)
    PictureIO.picture_io.get self.filename, subdir
  end
  
  def self.export(dt = nil)
    opt = {}
    cond = if dt
      ['artists.author_id is not null and pictures.updated_at >= ?', dt]
    else
      'artists.author_id is not null'
    end
    opt.merge!({:conditions => cond}) 
    opt.merge!({:include => {:artist => {}}, :order => 'pictures.updated_at desc'})
    Picture.find(:all, opt)
  end
  
  def self.list_as_json_text ary
    '[' + ary.map {|i| i.to_json_with_picture_data }.join(',') + ']'
  end
  
  def picture_data
    Base64.encode64(self.restore)
  end
  
  def to_json_with_picture_data
    self.to_json({:methods => :picture_data})
  end
  
  def unpublish
    imager = PettanImager.load(File.open(Rails.root + 'app/assets/images/error.png', 'rb').read)
    return false unless imager
    self.store imager
  end
  
  def credit_template
    "#{self.classname.tableize}/attributes/credit"
  end
  
  def full_credit_template
    "#{self.classname.tableize}/attributes/full_credit"
  end
  
  def credit_data
    begin
      @credit_data = JSON.parse(self.credit) unless @credit_data
    rescue 
    end
    @credit_data
  end
  
  def flags
    return @flags if @flags
    begin
      @flags = JSON.parse(self.settings)
    rescue 
    end
    @flags = {} unless @flags
    @flags
  end
  
  def flags=(s)
    @flags = s
  end
  
  def flag_open
    @flag_open = (flags["open"] || -1) unless @flag_open
    @flag_open
  end
  
  def flag_commercial
    @flag_commercial = (flags["commercial"] || -1) unless @flag_commercial
    @flag_commercial
  end
  
  def flag_official
    @flag_official = (flags["official"] || -1) unless @flag_official
    @flag_official
  end
  
  def flag_attribution
    @flag_attribution = (flags["attribution"] || -1) unless @flag_attribution
    @flag_attribution
  end
  
  def flag_derive
    @flag_derive = (flags["derive"] || -1) unless @flag_derive
    @flag_derive
  end
  
  def flag_thumbnail
    @flag_thumbnail = (flags["thumbnail"] || -1) unless @flag_thumbnail
    @flag_thumbnail
  end
  
  def flag_gif_convert
    @flag_gif_convert = (flags["gif_convert"] || -1) unless @flag_gif_convert
    @flag_gif_convert
  end
  
  def flag_reverse
    @flag_reverse = (flags["reverse"] || -1) unless @flag_reverse
    @flag_reverse
  end
  
  def flag_resize
    @flag_resize = (flags["resize"] || -1) unless @flag_resize
    @flag_resize
  end
  
  def flag_sync_vh
    @flag_sync_vh = (flags["sync_vh"] || -1) unless @flag_sync_vh
    @flag_sync_vh
  end
  
  def flag_overlap
    @flag_overlap = (flags["overlap"] || -1) unless @flag_overlap
    @flag_overlap
  end
  
end
