class SpeechBalloon < ActiveRecord::Base
  has_many :balloons, :dependent => :destroy
  has_many :speeches, :dependent => :destroy
  belongs_to :speech_balloon_template
  belongs_to :panel
  
  accepts_nested_attributes_for :balloons
  accepts_nested_attributes_for :speeches
  
  validates :panel_id, :numericality => {:allow_blank => true}
  validates :speech_balloon_template_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :classname, :presence => true, :length => {:maximum => 50}
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
#  validates :settings, :presence => true
  
  def visible? roles
    if MagicNumber['run_mode'] == 0
      return false unless guest_role_check(roles)
    else
      return false unless reader_role_check(roles)
    end
    return true if self.panel.own?(roles)
    self.panel.visible? roles
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(SpeechBalloon.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => 'panels.publish > 0', :order => 'speech_balloons.updated_at desc'})
    SpeechBalloon.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:panel => {:author => {}}, :balloons => {}, :speeches => {}, :speech_balloon_template => {} }}
  end
  
  def self.list_json_opt
    {:include => {:panel => {:include => {:author => {} }}, :balloons => {}, :speeches => {}, :speech_balloon_template => {} }}
  end
  
  def self.show cid, au
    opt = {}
    opt.merge!(SpeechBalloon.show_opt)
    res = SpeechBalloon.find(cid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(au)
    res
  end
  
  def self.show_opt
    {:include => {:panel => {:author => {}}, :balloons => {}, :speeches => {}, :speech_balloon_template => {} }}
  end
  
  def self.show_json_opt
    {:include => {:panel => {:include => {:author => {} }}, :balloons => {}, :speeches => {}, :speech_balloon_template => {} }}
  end
  
end
